package de.espirit.firstspirit.opt.example.editorialsar.clientservice;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.access.project.Project;
import de.espirit.firstspirit.access.store.Store;
import de.espirit.firstspirit.agency.ClientServiceRegistryAgent;
import de.espirit.firstspirit.agency.ModuleAdminAgent;
import de.espirit.firstspirit.agency.ProjectAgent;
import de.espirit.firstspirit.agency.StoreAgent;
import de.espirit.firstspirit.client.plugin.JavaClientPermanentPlugin;
import de.espirit.firstspirit.client.plugin.WebeditPermanentPlugin;
import de.espirit.firstspirit.io.FileSystem;
import de.espirit.firstspirit.opt.example.editorialsar.projectapp.EditorialSARProjectApp;
import de.espirit.firstspirit.opt.example.editorialsar.projectapp.EditorialSARProjectAppStorage;

import org.jetbrains.annotations.NotNull;


public class EditorialSARPermanentPlugin implements JavaClientPermanentPlugin, WebeditPermanentPlugin {

	private BaseContext _context;

	private EditorialSARClientService _clientService;


	/**
	 * Register {@link EditorialSARClientService} if {@link EditorialSARProjectApp} is available in the project.
	 *
	 * @param context The context the plug-in lives in.
	 */
	@Override
	public void setUp(@NotNull final BaseContext context) {
		_context = context;

		try {
			// Attempt to access the EditorialSAR project application. Only create and register the EditorialSARClientService as a
			// client service if the project app is installed for the currently loaded project. If the project application
			// is not available, an IllegalArgumentException will be thrown and handled instead of initializing the controller.
			final Project project = _context.requireSpecialist(StoreAgent.TYPE).getStore(Store.Type.PAGESTORE).getProject();
			final ModuleAdminAgent moduleAdminAgent = _context.requireSpecialist(ModuleAdminAgent.TYPE);
			final FileSystem<?> projectAppConfig = moduleAdminAgent.getProjectAppConfig(EditorialSARProjectApp.MODULE_NAME, EditorialSARProjectApp.PROJECT_APP_NAME, project);

			final EditorialSARProjectAppStorage projectAppStorage = new EditorialSARProjectAppStorage(_context, projectAppConfig);

			_clientService = new EditorialSARClientService(projectAppStorage, context);
			final ClientServiceRegistryAgent clientServiceRegistryAgent = _context.requireSpecialist(ClientServiceRegistryAgent.TYPE);
			clientServiceRegistryAgent.registerClientService(EditorialSARClientService.class, _clientService);
		} catch (final IllegalArgumentException e) {
			context.logInfo("Project app 'EditorialSARProjectApp' not found in project '" + context.requireSpecialist(ProjectAgent.TYPE).getName() + "' (ID: " + context.requireSpecialist(ProjectAgent.TYPE).getId() + "). Will not register Editorial Search and Replace client service.");
		}
	}


	/**
	 * Unregister {@link EditorialSARClientService}.
	 */
	@Override
	public void tearDown() {
		final ClientServiceRegistryAgent clientServiceRegistryAgent = _context.requireSpecialist(ClientServiceRegistryAgent.TYPE);
		clientServiceRegistryAgent.unregisterClientService(_clientService);
	}
}
