package de.espirit.firstspirit.opt.example.editorialsar.report.entry;

import de.espirit.firstspirit.access.store.mediastore.Media;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.storeelement.MediaStoreElementSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;


/**
 * Report-internal representation of a {@link Media} report entry.
 */
public class MediaReportEntry extends ReportEntry {

	private final Media _medium;


	/**
	 * Constructor.
	 *
	 * See {@link ReportEntry#ReportEntry(EditorialSARClientService, SearchConfiguration)} for the super-class constructor.
	 *
	 * @param clientService The {@link EditorialSARClientService}.
	 * @param searchConfiguration The {@link SearchConfiguration}.
	 * @param medium The {@link Media} element this report entry will represent.
	 */
	public MediaReportEntry(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final Media medium) {
		super(clientService, searchConfiguration);
		_medium = medium;
	}


	/**
	 * See {@link ReportEntry#getElement()}.
	 */
	@NotNull
	@Override
	public Media getElement() {
		return _medium;
	}


	/**
	 * See {@link ReportEntry#getWebeditTargetElement()}.
	 *
	 * {@link Media} elements shouldn't be shown individually in ContentCreator's preview, so {@code null} is returned.
	 */
	@Nullable
	@Override
	public Media getWebeditTargetElement() {
		return null;
	}


	/**
	 * See {@link ReportEntry#doSearch()}.
	 */
	@Override
	public int doSearch() throws IllegalStateException {
		if (_searchConfiguration.isValidForSearch()) {
			_results.clear();
			int count = 0;
			final MediaStoreElementSearchAdapter mediaSearchAdapter = new MediaStoreElementSearchAdapter(_clientService, _searchConfiguration, _medium);
			_results.addAll(mediaSearchAdapter.search());
			for (final SearchResult result : _results) {
				count += result.getResultCount();
			}
			return count;
		} else {
			throw new IllegalStateException("Attempt to run a search without valid search settings.");
		}
	}


	/**
	 * See {@link ReportEntry#doReplace()}.
	 */
	@Override
	public int doReplace() throws IllegalStateException {
		if (_searchConfiguration.isValidForReplace()) {
			int count = 0;
			final MediaStoreElementSearchAdapter mediaSearchAdapter = new MediaStoreElementSearchAdapter(_clientService, _searchConfiguration, _medium);
			for (final SearchResult searchResult : mediaSearchAdapter.replace()) {
				count += searchResult.getResultCount();
			}
			return count;
		} else {
			throw new IllegalStateException("Attempt to run a replace without valid search settings.");
		}
	}
}