package de.espirit.firstspirit.opt.example.editorialsar.report.entry;

import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.access.store.templatestore.TemplateStoreElement;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.storeelement.TemplateStoreElementSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;


/**
 * Report-internal representation of a {@link TemplateStoreElement} report entry.
 */
public class TemplateReportEntry extends ReportEntry {

	private final TemplateStoreElement _template;


	/**
	 * Constructor.
	 *
	 * See {@link ReportEntry#ReportEntry(EditorialSARClientService, SearchConfiguration)} for the super-class constructor.
	 *
	 * @param clientService The {@link EditorialSARClientService}.
	 * @param searchConfiguration The {@link SearchConfiguration}.
	 * @param template A template element based on the interface {@link TemplateStoreElement} or a sub-class thereof.
	 */
	public TemplateReportEntry(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final TemplateStoreElement template) {
		super(clientService, searchConfiguration);
		_template = template;
	}


	/**
	 * See {@link ReportEntry#getElement()}.
	 */
	@NotNull
	@Override
	public TemplateStoreElement getElement() {
		return _template;
	}


	/**
	 * See {@link ReportEntry#getWebeditTargetElement()}.
	 *
	 * As templates are not viewable in the ContentCreator preview, this method will always return {@code null}.
	 */
	@Nullable
	@Override
	public IDProvider getWebeditTargetElement() {
		return null;
	}


	/**
	 * See {@link ReportEntry#doSearch()}.
	 */
	@Override
	public int doSearch() throws IllegalStateException {
		if (_searchConfiguration.isValidForSearch()) {
			_results.clear();
			int count = 0;
			final TemplateStoreElementSearchAdapter templateSearchAdapter = new TemplateStoreElementSearchAdapter(_clientService, _searchConfiguration, _template);
			_results.addAll(templateSearchAdapter.search());
			for (final SearchResult result : _results) {
				count += result.getResultCount();
			}
			return count;
		} else {
			throw new IllegalStateException("Attempt to run a search without valid search settings.");
		}
	}


	/**
	 * See {@link ReportEntry#doReplace()}.
	 */
	@Override
	public int doReplace() throws IllegalStateException {
		if (_searchConfiguration.isValidForReplace()) {
			int count = 0;
			final TemplateStoreElementSearchAdapter templateSearchAdapter = new TemplateStoreElementSearchAdapter(_clientService, _searchConfiguration, _template);
			for (final SearchResult searchResult : templateSearchAdapter.replace()) {
				count += searchResult.getResultCount();
			}
			return count;
		} else {
			throw new IllegalStateException("Attempt to run a replace without valid search settings.");
		}
	}
}
