package de.espirit.firstspirit.opt.example.editorialsar.report.item;

import de.espirit.firstspirit.access.store.IDProvider;
import de.espirit.firstspirit.agency.OperationAgent;
import de.espirit.firstspirit.client.plugin.report.ReportContext;
import de.espirit.firstspirit.opt.example.editorialsar.report.entry.ReportEntry;
import de.espirit.firstspirit.ui.operations.DisplayElementOperation;
import de.espirit.firstspirit.webedit.plugin.report.WebeditExecutableReportItem;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ResourceBundle;


/**
 * Report item for ContentCreator that provides JavaScript to perform navigation to a report entry's element (or a related target element) on-demand.
 */
public class NavigateToElementWebeditExecutableReportItem implements WebeditExecutableReportItem<ReportEntry> {

	private final ResourceBundle _resourceBundle;


	public NavigateToElementWebeditExecutableReportItem(@NotNull final ResourceBundle resourceBundle) {
		_resourceBundle = resourceBundle;
	}


	@NotNull
	@Override
	public String getLabel(@NotNull final ReportContext<ReportEntry> context) {
		return _resourceBundle.getString("report.item.navigate_to.label");
	}


	@Override
	public void execute(@NotNull final ReportContext<ReportEntry> context) {
		// Obtain a ContentCreator-compatible target element.
		final IDProvider jumpTargetElement = context.getObject().getWebeditTargetElement();

		// Cause navigation to the target element.
		if (jumpTargetElement != null) {
			final OperationAgent operationAgent = context.requireSpecialist(OperationAgent.TYPE);
			final DisplayElementOperation displayElementOperation = operationAgent.getOperation(DisplayElementOperation.TYPE);
			if (displayElementOperation != null) {
				displayElementOperation.setLanguage(context.getObject().getResults().get(0).getLanguage());
				displayElementOperation.perform(jumpTargetElement);
			}
		}
	}


	@Nullable
	@Override
	public String getIconPath(@NotNull final ReportContext<ReportEntry> context) {
		return "mod-editorialsar/media/item_NavigateToElement_normal.png";
	}


	@Override
	public boolean isVisible(@NotNull final ReportContext<ReportEntry> context) {
		return context.getObject().getWebeditTargetElement() != null;
	}


	@Override
	public boolean isEnabled(@NotNull final ReportContext<ReportEntry> context) {
		return true;
	}
}