package de.espirit.firstspirit.opt.example.editorialsar.search.adapter;

import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;

import java.util.List;


/**
 * Search adapter to perform search and replacement processes on specific data types.
 */
public abstract class SearchAdapter {

	protected final EditorialSARClientService _clientService;
	protected final SearchConfiguration _searchConfiguration;

	protected boolean _replace;


	protected SearchAdapter(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration) {
		_clientService = clientService;
		_searchConfiguration = searchConfiguration;
	}


	/**
	 * Perform a search across the entire given data domain.
	 *
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	public List<SearchResult> search() {
		_replace = false;
		return process();
	}


	/**
	 * Perform a replacement, replacing all found occurrences in the given data domain.
	 *
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	public List<SearchResult> replace() {
		_replace = true;
		return process();
	}


	/**
	 * Process a data domain, using the given parameters as processing settings.
	 *
	 * @param replace Perform a replacement?
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	public List<SearchResult> process(final boolean replace) {
		_replace = replace;
		return process();
	}


	/**
	 * Process a data domain.
	 *
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	protected abstract List<SearchResult> process();
}