package de.espirit.firstspirit.opt.example.editorialsar.search.adapter.gomelement;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.editor.value.Table;
import de.espirit.firstspirit.access.editor.value.TableDomNode;
import de.espirit.firstspirit.forms.FormData;
import de.espirit.firstspirit.forms.FormField;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.SearchAdapterUtil;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.FormDataSearchResult;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


/**
 * Search adapter to perform search and replacement processes on {@link Table} objects.
 */
public class DomTableGomElementSearchAdapter extends GomElementSearchAdapter {


	public DomTableGomElementSearchAdapter(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final Language language, @NotNull final FormData formData, final boolean isMetadataForm, @NotNull final String editorName) {
		super(clientService, searchConfiguration, language, formData, isMetadataForm, editorName);
	}


	/**
	 * Process a {@link FormField} containing {@link TableDomNode} (CMS_INPUT_DOMTABLE) data.
	 *
	 * @return A {@link List} of {@link SearchResult} objects.
	 */
	@NotNull
	@Override
	protected List<SearchResult> process() {
		final FormField<?> formField = _formData.get(_language, _editorName);
		if (formField.getType().equals(Table.class)) {
			final TableDomNode tableDomNode = (TableDomNode) formField.get();
			final List<SearchResult> searchResults = new ArrayList<SearchResult>();
			if (tableDomNode != null) {
				// Send the table editor's root DOM node to SearchAdapterUtil.processDomNode(...).
				final List<SearchAdapterUtil.ProcessingResult> processingResults = SearchAdapterUtil.processDomNode(_clientService, _searchConfiguration, tableDomNode, _replace);
				int processingCount = 0;
				boolean replacementOccurred = false;
				for (final SearchAdapterUtil.ProcessingResult processingResult : processingResults) {
					processingCount += processingResult.getProcessingCount();
					replacementOccurred |= processingResult.getReplacementStatus();
				}
				if (processingCount > 0) {
					searchResults.add(new FormDataSearchResult(_editorName, _language, _isMetadataForm, processingCount));
					if (_replace && replacementOccurred) {
						formField.set(tableDomNode);
					}
				}
				return searchResults;
			}
		}
		return Collections.emptyList();
	}
}
