package de.espirit.firstspirit.opt.example.editorialsar.search.adapter.gomelement;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.editor.fslist.IdProvidingFormData;
import de.espirit.firstspirit.access.store.templatestore.SectionTemplate;
import de.espirit.firstspirit.access.store.templatestore.TableTemplate;
import de.espirit.firstspirit.access.store.templatestore.Template;
import de.espirit.firstspirit.forms.FormData;
import de.espirit.firstspirit.forms.FormDataList;
import de.espirit.firstspirit.forms.FormField;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.formdata.FormDataSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;


/**
 * Search adapter to perform search and replacement processes on {@link FormDataList} objects.
 */
public class FsListGomElementSearchAdapter extends GomElementSearchAdapter {


	public FsListGomElementSearchAdapter(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final Language language, @NotNull final FormData formData, final boolean isMetadataForm, @NotNull final String editorName) {
		super(clientService, searchConfiguration, language, formData, isMetadataForm, editorName);
	}


	/**
	 * Process a {@link FormField} containing {@link FormDataList} (FS_LIST) data.
	 *
	 * Only list entries with a template of type SectionTemplate (with exclusion of table templates) will be sent to search and replace processing.
	 *
	 * @return A {@link List} of {@link SearchResult} objects.
	 */
	@NotNull
	@Override
	protected List<SearchResult> process() {
		final List<SearchResult> searchResults = new ArrayList<SearchResult>();
		final FormField<?> formField = _formData.get(_language, _editorName);
		if (formField.getType().equals(FormDataList.class)) {
			final FormDataList formDataList = (FormDataList) formField.get();
			if (formDataList != null) {
				// Process each list entry.
				for (final FormData formData : formDataList) {
					if (formData instanceof IdProvidingFormData) {
						final Template template = ((IdProvidingFormData) formData).getTemplate();

						// This search adapter only considers FS_LIST inline sections (list entries with a template of type SectionTemplate).
						// Because TableTemplate is an extension of SectionTemplate, we need to exclude entries with a template of type TableTemplate!
						if (template instanceof SectionTemplate && !(template instanceof TableTemplate) && _clientService.isSupportedTemplate(template)) {
							// The template is valid for search and replace.
							// Process the list entry using a FormDataSearchAdapter.
							final FormDataSearchAdapter formDataSearchAdapter = new FormDataSearchAdapter(_clientService, _searchConfiguration, _language, formData, _isMetadataForm);
							searchResults.addAll(formDataSearchAdapter.process(_replace));
						}
					}
				}
			}
		}
		return searchResults;
	}
}