package de.espirit.firstspirit.opt.example.editorialsar.search.adapter.gomelement;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.forms.FormData;
import de.espirit.firstspirit.forms.FormField;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.SearchAdapterUtil;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.FormDataSearchResult;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;

import java.util.Collections;
import java.util.List;


/**
 * Search adapter to perform search and replacement processes on {@link String}-based {@link FormField} values.
 */
public class TextGomElementSearchAdapter extends GomElementSearchAdapter {


	public TextGomElementSearchAdapter(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final Language language, @NotNull final FormData formData, final boolean isMetadataForm, @NotNull final String editorName) {
		super(clientService, searchConfiguration, language, formData, isMetadataForm, editorName);
	}


	/**
	 * Process a {@link FormField} containing simple {@link String} (e.g. CMS_INPUT_TEXT, CMS_INPUT_TEXTAREA) data.
	 *
	 * @return A {@link List} of {@link SearchResult} objects.
	 */
	@NotNull
	@Override
	protected List<SearchResult> process() {
		final FormField<?> formField = _formData.get(_language, _editorName);
		if (formField.getType().equals(String.class)) {
			final String editorValue = (String) formField.get();
			if (editorValue != null && !editorValue.isEmpty()) {
				final SearchAdapterUtil.ProcessingResult result = SearchAdapterUtil.processText(editorValue, _searchConfiguration, _replace);
				if (result.getProcessingCount() > 0) {
					if (_replace && result.getReplacementStatus()) {
						formField.set(result.getProcessedText());
					}
					return Collections.<SearchResult>singletonList(new FormDataSearchResult(_editorName, _language, _isMetadataForm, result.getProcessingCount()));
				}
			}
		}
		return Collections.emptyList();
	}
}