package de.espirit.firstspirit.opt.example.editorialsar.search.adapter.storeelement;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.editor.EditorValue;
import de.espirit.firstspirit.access.editor.TableTemplateProvider;
import de.espirit.firstspirit.access.store.pagestore.DataProvider;
import de.espirit.firstspirit.access.store.templatestore.TemplateProvider;
import de.espirit.firstspirit.forms.FormData;
import de.espirit.firstspirit.opt.example.editorialsar.clientservice.EditorialSARClientService;
import de.espirit.firstspirit.opt.example.editorialsar.search.SearchConfiguration;
import de.espirit.firstspirit.opt.example.editorialsar.search.adapter.formdata.FormDataSearchAdapter;
import de.espirit.firstspirit.opt.example.editorialsar.search.result.SearchResult;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


/**
 * Search adapter to perform search and replacement processes on elements of type {@link DataProvider}.
 */
public class DataProviderStoreElementSearchAdapter extends StoreElementSearchAdapter {


	public DataProviderStoreElementSearchAdapter(@NotNull final EditorialSARClientService clientService, @NotNull final SearchConfiguration searchConfiguration, @NotNull final DataProvider element) {
		super(clientService, searchConfiguration, element);
	}


	/**
	 * Process the store element, regarding replacement and single-stepping settings.
	 *
	 * @return A {@link List} of {@link SearchResult}s.
	 */
	@NotNull
	@Override
	protected List<SearchResult> process() {
		final List<SearchResult> searchResults = new ArrayList<SearchResult>();

		// Check if the element is of type DataProvider and if the element's template is accepted for search and replace operations (according to project application configuration).
		if ((_element instanceof DataProvider) &&
				((_element instanceof TemplateProvider && _clientService.isSupportedTemplate(((TemplateProvider<?>) _element).getTemplate())) ||
						(_element instanceof TableTemplateProvider && ((TableTemplateProvider) _element).getTableTemplate() != null && _clientService.isSupportedTemplate(((TableTemplateProvider) _element).getTableTemplate())))
				) {

			// Obtain the form data object and process it using a FormDataSearchAdapter instance.
			final FormData formData = ((DataProvider) _element).getFormData();

			final List<Language> languages = _searchConfiguration.getLanguages();
			for (final Language language : languages) {
				final FormDataSearchAdapter formDataSearchAdapter = new FormDataSearchAdapter(_clientService, _searchConfiguration, language, formData, false);
				searchResults.addAll(formDataSearchAdapter.process(_replace));
			}

			// If this is a replacement operation, reassign the form data object to the element for persistence.
			if (_replace) {
				((DataProvider) _element).setFormData(formData);
			}

			// If the element has metadata, the user may change metadata, and the metadata template is supported, process that data using a FormDataSearchAdapter.
			if (_element.hasMeta() && _element.getPermission().canMetaChange() && _clientService.isMetadataTemplateSupported()) {
				final FormData metaFormData = _element.getMetaFormData();
				if (metaFormData != null) {
					final FormDataSearchAdapter metaFormDataSearchAdapter = new FormDataSearchAdapter(_clientService, _searchConfiguration, EditorValue.SOLE_LANGUAGE, metaFormData, true);
					searchResults.addAll(metaFormDataSearchAdapter.process(_replace));
					// If this is a replacement operation, reassign the metadata form data object to the element for persistence.
					if (_replace) {
						_element.setMetaFormData(metaFormData);
					}
				}
			}

		} else {
			return Collections.emptyList();
		}

		return searchResults;
	}
}