package de.espirit.firstspirit.opt.example.editorialsar.search.result;

import de.espirit.firstspirit.access.Language;

import org.jetbrains.annotations.NotNull;


/**
 * {@link SearchResult} implementation for form data.
 */
public class FormDataSearchResult implements SearchResult {

	private final boolean _isMetadataForm;
	private final String _editorName;
	private final Language _language;
	private final int _resultCount;


	/**
	 * Constructor.
	 *
	 * @param editorName The GOM editor name this search result relates to.
	 * @param language The project {@link Language}.
	 * @param isMetadataForm Indicates if the result relates to metadata.
	 * @param resultCount The number of times the search string has been identified in the editor.
	 */
	public FormDataSearchResult(@NotNull final String editorName, @NotNull final Language language, final boolean isMetadataForm, final int resultCount) {
		_editorName = editorName;
		_language = language;
		_isMetadataForm = isMetadataForm;
		_resultCount = resultCount;
	}


	@NotNull
	public String getEditorName() {
		return _editorName;
	}


	@NotNull
	@Override
	public Language getLanguage() {
		return _language;
	}


	public boolean isMetadataForm() {
		return _isMetadataForm;
	}


	@Override
	public int getResultCount() {
		return _resultCount;
	}
}
