package de.espirit.firstspirit.opt.example.editorialsar.search.result;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.store.mediastore.Media;

import org.jetbrains.annotations.NotNull;


/**
 * {@link SearchResult} implementation for {@link Media} elements.
 */
public class MediaSearchResult implements SearchResult {

	private final Field _field;
	private final Language _language;
	private final int _resultCount;


	/**
	 * Indicates a specific data field that can be stored in a {@link Media} element.
	 */
	public enum Field {
		FILENAME,
		DESCRIPTION,
	}


	/**
	 * Constructor.
	 *
	 * @param field The {@link Field} this search result relates to.
	 * @param language The project {@link Language}.
	 * @param resultCount The number of times the search string has been identified in the given {@link Field}.
	 */
	public MediaSearchResult(@NotNull final Field field, @NotNull final Language language, final int resultCount) {
		_field = field;
		_language = language;
		_resultCount = resultCount;
	}


	@NotNull
	public Field getField() {
		return _field;
	}


	@NotNull
	@Override
	public Language getLanguage() {
		return _language;
	}


	@Override
	public int getResultCount() {
		return _resultCount;
	}
}
