package de.espirit.firstspirit.opt.example.editorialsar.search.result;

import de.espirit.firstspirit.access.Language;
import de.espirit.firstspirit.access.editor.EditorValue;
import de.espirit.firstspirit.access.project.TemplateSet;
import de.espirit.firstspirit.access.store.templatestore.TemplateStoreElement;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;


/**
 * {@link SearchResult} constructor for {@link TemplateStoreElement}s.
 */
public class TemplateSearchResult implements SearchResult {

	private final Tab _tab;
	private final TemplateSet _templateSet;
	private final int _resultCount;


	/**
	 * Indicates a specific template tab that can be stored in most {@link TemplateStoreElement}s.
	 */
	public enum Tab {
		PROPERTIES,
		GOM_SOURCE,
		RULESET_DEFINITION,
		SNIPPET,
		CHANNEL
	}


	/**
	 * Constructor to be used for template tabs other than output channels.
	 *
	 * @param tab The template tab this search result relates to.
	 * @param resultCount The number of times the search string has been identified in the template tab.
	 */
	public TemplateSearchResult(@NotNull final Tab tab, final int resultCount) throws IllegalStateException {
		if (tab == Tab.CHANNEL) {
			throw new IllegalStateException("Illegal use of " + getClass() + " constructor: used Tab.CHANNEL without TemplateSet");
		}
		_tab = tab;
		_templateSet = null;
		_resultCount = resultCount;
	}


	/**
	 * Constructor to be used for output channels.
	 *
	 * @param templateSet The {@link TemplateSet} (i.e. output channel) this search result relates to.
	 * @param resultCount The number of times the search string has been identified in the template set.
	 */
	public TemplateSearchResult(@NotNull final TemplateSet templateSet, final int resultCount) {
		_tab = Tab.CHANNEL;
		_templateSet = templateSet;
		_resultCount = resultCount;
	}


	@NotNull
	public Tab getTab() {
		return _tab;
	}


	@Nullable
	public TemplateSet getTemplateSet() {
		if (_tab == Tab.CHANNEL) {
			return _templateSet;
		}
		return null;
	}


	@NotNull
	@Override
	public Language getLanguage() {
		return EditorValue.SOLE_LANGUAGE;
	}


	@Override
	public int getResultCount() {
		return _resultCount;
	}
}
